#' @title Adapt eaip data
#'
#' @description
#' The following treatment are performed:
#' - all files for a given department are grouped (through a sf rbind)
#' - only some common variables are kept
#' - the projection is set to projection
#' - the geometry is made valid
#' - the output is saved at department level as rds in given destination
#'
#' @param origin character, path to the directory where eaip are stored
#' @param destination character, path to the directory where results should be saved
#' @param department vector, departments to be adpated
#' @param export character, option for export format of the results
#' @param projection integer or object of type CRS giving projection
#' @param variable_common character, what are the variables that should be kept. Best to leave it to default or NULL.
#' @param validity logical, should st_make_valid be applied (may be very long).
#' @param retrieve logical, should the result be returned
#' @param verbose logical, should the function give some sumup informations.
#'
#' @return if retrieve = TRUE, sf object representing eaip for asked departments.
#'
#' @export
#'
#' @encoding UTF-8
#' @author Frédéric Grelot
#' 
#' @examples
#' 
#' \dontrun{
#' origin = "~/data/BD/EAIP/shp"
#' destination = tempdir()
#' eaip_01 = adapt.eaip(origin, destination, department = 1, verbose = TRUE, retrieve = TRUE)
#' }

adapt.eaip = function(origin,
    destination,
    department,
    export = "rds",
    projection = 4326,
    variable_common = floodam.data::eaip_variable,
    validity = TRUE,
    retrieve = FALSE,
    verbose = FALSE) {

    ### Analyse department
    if (missing(department)) {
        shp = dir(origin, "[.]shp$")
        department = sort(unique(unlist(regmatches(shp, regexec("(?<=d)[0-9AB]{2,3}(?=_)", shp, perl = TRUE)))))
    }
    if (length(department) > 1) 
        return(lapply(department, adapt.eaip,
            origin = origin,
            destination = destination,
            export = export,
            projection = projection,
            variable_common = variable_common,
            validity = validity,
            retrieve  = retrieve,
            verbose = verbose))
    if (length(department) == 0)
        return(NULL)
    
    ### Start journal information
    dir.create(destination, showWarnings = FALSE, recursive = TRUE) # Check if error may be generated.
    start_time = Sys.time()
    journal = add_journal_old("WARNING: This file is automatically produced by floodam.data. Do not edit by hand.", "")

    shp = dir(origin, "[.]shp$")
    if (! format_scope(department, type = "insee") %in% floodam.data::department[["code"]]) {
        journal = add_journal_old(
            sprintf("'%s' is not a good format for department. Nothing is done.", department),
            verbose = TRUE, first = TRUE
        )
        return(NULL)
    }
    department = format_scope(department, type = "insee")
    archive = sprintf("eaip_%s", format_scope(department))
    
    ### Preliminary tests
    if (length(shp) == 0) {
        journal = add_journal_old("No shapefiles found.", verbose = TRUE, first = TRUE)
        return(NULL)
    }
    shp = grep(sprintf("d%2s", department), shp, value = TRUE)
    if (length(shp) == 0) {
        journal = add_journal_old(
            sprintf("Nothing corresponding to department '%s' found in %s", department, origin),
            verbose = TRUE, first = TRUE
        )
        return(NULL)
    }

    ### Add journal information
    journal = add_journal_old(
        sprintf("Department %s treated through adaptation of %s from %s by floodam.data (%s)", 
            department, paste(shp, collapse = ", "),
            origin, utils::packageVersion("floodam.data")),
        journal = journal, verbose = verbose, first = TRUE)
    on.exit(journal <- add_journal_old(
        sprintf("Time elapsed: %s", fmt(Sys.time() - start_time)),
        journal = journal, verbose = verbose))
    on.exit(write(journal, file.path(destination, sprintf("%s.log", archive))), add = TRUE)

    ### Read data
    input = lapply(file.path(origin, shp), sf::st_read, quiet = TRUE)
    
    for (i in seq_along(shp)) {
    ### Check projection
        if (is.na(sf::st_crs(input[[i]]))) {
            sf::st_crs(input[[i]]) = 2154
            journal = add_journal_old(
                sprintf("%s has no projection. Set to EPSG:2154.", shp[i]),
                journal = journal, verbose = verbose)            
        }

    ### Transform projection       
        if (!is.null(projection)) {
            temp = try(sf::st_crs(projection), silent = TRUE)
            projection = if ("crs" %in% class(temp)) temp else projection
            if ("crs" %in% class(projection)) {
                input[[i]] = sf::st_transform(input[[i]], crs = projection)
                journal = add_journal_old(
                    sprintf("projection %s applied to %s.", projection[["input"]], shp[i]),
                    journal = journal, verbose = verbose)
            } else {
                journal = add_journal_old(
                    sprintf("projection %s could not be applied to %s.", projection, shp[i]),
                    journal = journal, verbose = verbose)
            }
        }

    ### Some corrections on variable names
        names(input[[i]])[names(input[[i]]) == "APPROBATIO"] = "DATE_APPRO"
        names(input[[i]])[names(input[[i]]) == "FOURNISSEU"] = "FOURNISS"
        names(input[[i]])[names(input[[i]]) == "IMPORT"] = "DATE_IMPOR"
    }

    ### Add all shp in same geometry
    if (is.null(variable_common)) {
        variable = Reduce(intersect, c(lapply(input, names)))
    } else {
        variable = Reduce(intersect, c(lapply(input, names), list(variable_common)))
    }
    result = do.call(rbind, lapply(input, "[", variable))
    variable_original = unique(unlist(lapply(input, names)))
    journal = add_journal_old(
        sprintf("%s shapefile(s) combined. %s variable(s) discarded for consistency. %s variable(s) kept.", 
            length(input), max(sapply(input, ncol) - length(variable)), length(variable)),
        journal = journal, verbose = verbose)
    if (length(variable) < length(variable_original)) {
        journal = add_journal_old(
            sprintf("present variable(s) discarded: %s.", 
                paste(variable_original[! variable_original %in% variable], collapse = ", ")),
            journal = journal, verbose = verbose)        
    }
    if (!is.null(variable_common)) {
        discarded_common = variable_common[! variable_common %in% variable]
        if (length(discarded_common) > 0) {
            journal = add_journal_old(
                sprintf("WARNING: common variable(s) discarded: %s. Added with NA values.", 
                    paste(discarded_common, collapse = ", ")),
                journal = journal, verbose = verbose)
            result[discarded_common] = NA
        }
        result = result[variable_common]
    }
    
    ### Make geometry valid
    if (validity == TRUE) {
        area_initial = sum(sf::st_area(result))
        n_initial = nrow(result)
        result = suppressWarnings(sf::st_cast(result, "POLYGON"))
        result = sf::st_make_valid(result)
        area_final = sum(sf::st_area(result))
        area_diff = area_final - area_initial
        n_final = nrow(result)
        journal = add_journal_old("Geometry validity applied.", journal = journal, verbose = verbose)
        journal = add_journal_old(
            sprintf("\t%s original entities divided in %s polygons", n_initial, n_final),
            journal = journal)
        journal = add_journal_old(
            sprintf("\tinitial area: \t%+14.2f\t(%s)",
                area_initial, as.character(attr(area_initial, "units"))),
            journal = journal)
        journal = add_journal_old(
            sprintf("\tfinal area:   \t%+14.2f\t(%s)",
                area_final, as.character(attr(area_final, "units"))),
            journal = journal)
        journal = add_journal_old(
            sprintf("\tdiff of area: \t%+14.2f\t(%s)",
                area_diff, as.character(attr(area_diff, "units"))),
            journal = journal)
        journal = add_journal_old(
            sprintf("\trelative diff:\t%+14.2f",
                area_diff / area_initial),
            journal = journal)
    }

    ### Save result, print journal, retrieve result
    if (identical(export, "rds")) {
        saveRDS(result, file.path(destination, sprintf("%s.%s", archive, export)))
        journal = add_journal_old(
            sprintf("saved as %s in %s.", sprintf("%s.%s", archive, export), destination),
            journal = journal, verbose = verbose)
    } else {
        journal = add_journal_old(
            sprintf("%s not admissible as export: adaptation not saved.", export),
            journal = journal, verbose = verbose)
    }

    if (retrieve) return(invisible(result))
}
