#' @title Adapt Admin Express archives
#'
#' @description
#' `adapt_admin_express()` treats Admin Express archives to be in a standard
#' format. It is suited for IGN archives. It used by default
#' \code{scheme_admin_express_3_1} as scheme. For older versions
#' \code{scheme_admin_express_2_0} is also available.
#' 
#' Beware that code{adapt_admin_express} will perform all possibles combinations
#' of archives present in origin and layers available in scheme if those
#' parameters are not specified.
#' 
#' @details
#' The following treatment are performed:
#' 
#' - depending on how many archives and layers are to be treated, a recursive
#' call may be perform to do all desired combinations.
#' - archive is read with [read_with_scheme()], which may imply that it is
#' uncompressed, then read (usually as a shp or a gpkg file), reprojected if
#' asked, and finally adapted.
#' - the output is saved as rds in given destination withe[save_archive()]. The
#' name of archive include a precision depending on the layer chosen. For layer
#' 'COMMUNE', the saving is made by departments.
#'
#' @param origin character, path to the directory where archive are stored.
#' @param destination character, path to the directory where results should be
#' saved.
#' @param archive character, vector of archive to be adpated.
#' @param layer character, which layer should be treated.
#' @param scheme data.frame, how archive variables should be treated.
#' @param projection integer or object of type CRS giving projection.
#' @param extension character, option for extension and so format of saving of
#' results.
#' @param journal logical, should a journal file be saved.
#' @param retrieve logical, should the result be returned.
#' @param verbose logical, should the function give some sumup informations.
#'
#' @return if retrieve = TRUE, sf object of adapated data stored in archive.
#' This may be a list if the function implies recursive calls.
#'
#' @export
#'
#' @encoding UTF-8
#' @author Frédéric Grelot and Victor Champonnois
#' 
#' @examples
#' 
#' \dontrun{
#' origin = ""
#' destination = tempdir()
#' archive = "RPG_2-0__SHP_LAMB93_FR-2018_2018-01-15.7z"
#' result = adapt_rpg(archive, origin, destination, verbose = TRUE, retrieve = TRUE)
#' }

adapt_admin_express = function(
    origin,
    destination,
    archive,
    layer = NULL,
    scheme = floodam.data::scheme_admin_express_3_1,
    projection = 4326,
    extension = "rds",
    journal = TRUE,
    retrieve = FALSE,
    verbose = TRUE
) {

    # If archive is missing, treat all archives in origin
    if (missing(archive)) {
        archive = list.files(origin, pattern = ".(7z.001|7z|zip)$")
    }
    # Recursive call if archive is lengthy
    if (length(archive) > 1) 
        return(
            lapply(
                archive,
                adapt_admin_express,
                origin = origin,
                layer = layer,
                destination = destination,
                scheme = scheme,
                projection = projection,
                extension = extension,
                journal = journal,
                retrieve = retrieve,
                verbose = verbose
            )
        )
    if (length(archive) == 0) return(NULL)

    # Recursive call if layer is lengthy
    if (is.null(layer)) layer = unique(scheme[["layer"]])
    layer = layer[layer %in% unique(scheme[["layer"]])]
    if (length(layer) > 1) 
        return(
            lapply(
                layer,
                adapt_admin_express,
                origin = origin,
                destination = destination,
                archive = archive,
                scheme = scheme,
                projection = projection,
                extension = extension,
                journal = journal,
                retrieve = retrieve,
                verbose = verbose
            )
        )
    if (length(layer) == 0) return(NULL)

    # Analyse archive
    archive = file.path(origin, archive)
    message(sprintf("'Admin Express' adaptation from '%s'...\n", basename(archive)))
    old_s2 = suppressMessages(sf::sf_use_s2(FALSE))
    on.exit(suppressMessages(sf::sf_use_s2(old_s2)))
    analyse = analyse_archive(basename(archive))
    dir.create(destination, showWarnings = FALSE, recursive = TRUE)

    # Start journal
    if (journal == TRUE) {
        journal = file.path(
            destination,
            sprintf("%s.log", basename_core(archive))
        )
        init_log_info(
            journal,
            main = sprintf("%s in %s", basename(archive), origin),
            treatment = "adapt_admin_express",
        )
    } else {
        journal = FALSE
    }

    # Check analyse
    if (is.na(analyse["data"])) {
        add_log_info(
            "Bad format for archive. Nothing is done.",
            message,
            info  = list(
                "short" = "skip analysis",
                "long" = "Skip analysis: archive name with bad format."
            ),
            journal = journal, verbose = verbose
        )
        return(NULL)
    }

    # Reading archive with scheme information
    result = add_log_info(
        archive,
        read_with_scheme,
        info  = list(
            "short" = "read 'admin express'",
            "long" = "Reading original 'admin express' archive and applying scheme"
        ),
        journal = journal, verbose = verbose,
        verification = expression(info_dim(result)),
        scheme = scheme,
        projection = projection,
        layer = layer
    )

    # Save data
    ## Prepare parameters
    precision = switch(
        layer,
        DEPARTEMENT = "department",
        COMMUNE = "commune",
        EPCI = "epci",
        REGION = "region",
        NA
    )
    scope = switch(
        precision,
        commune = "department",
        NULL
    )
    analyse["precision"] = precision
    analyse["extension"] = extension
    test = !is.na(analyse[["vintage"]]) &&
        !grepl(analyse["vintage"], destination)
    if (test) {destination = file.path(destination, analyse[["vintage"]])}
    dir.create(destination, showWarnings = FALSE, recursive = TRUE)
    
    add_log_info(result, save_archive,
        info  = list(
            "short" = "save",
            "long" = sprintf("Saving %s layer", layer)
        ),
        journal = journal, verbose = verbose,
        verification = expression(object.size(x)),
        path = file.path(
            destination,
            format_archive(analyse)
        ),
        scope = scope
    )
    
    # Retrieve result
    if (retrieve) {
        return(invisible(result))
    }
}