#' @title Adapt rpg data
#'
#' @description
#' The following treatment are performed:
#' - the projection is set to projection
#' - the geometry is made valid
#' - flood exposition is set through intersection with eaip
#' - the output is saved at department level as rds in given destination
#' 
#' @param origin character, path to the directory where archive are stored
#' @param destination character, path to the directory where results should be saved
#' @param archive character, vector of archive to be adpated
#' @param export character, option for export format of the results
#' @param scheme data.frame, how archive variables should be treated
#' @param department character, vector of department that should be treated
#' @param dep_path character, path to the directory where departments data are stored
#' @param eaip_path character, path where eaip archives should be find
#' @param projection integer or object of type CRS giving projection
#' @param retrieve logical, should the result be returned
#' @param verbose logical, should the function give some sumup informations.
#'
#' @return if retrieve = TRUE, sf object of adapated data stored in archive.
#'
#' @export
#'
#' @encoding UTF-8
#' @author Victor Champonnois et Frédéric Grelot
#' 
#' @examples
#' 
#' \dontrun{
#' origin = ""
#' destination = tempdir()
#' archive = "RPG_2-0__SHP_LAMB93_FR-2018_2018-01-15.7z"
#' result = adapt_rpg(archive, origin, destination, verbose = TRUE, retrieve = TRUE)
#' }

adapt_rpg = function(origin,
    destination,
    archive,
    export = "rds",
    scheme = NULL,
    department = NULL,
    dep_path = NULL,
    eaip_path = NULL,
    projection = 4326,
    retrieve = FALSE,
    verbose = FALSE) {

    # Recursive call if archive is missing
    if (missing(archive)) {archive = list.files(origin, pattern = ".(7z.001|7z|zip)$")}
    if (length(archive) > 1) 
        return(lapply(archive,
            adapt_rpg,
            origin = origin,
            destination = destination,
            export = export,
            scheme = scheme,
            department = department,
            dep_path = dep_path,
            eaip_path = eaip_path,
            projection = projection,
            retrieve = retrieve,
            verbose = verbose))
    if (length(archive) == 0)
        return(NULL)

    # Analyse archive. Skip if format is not good.
    analyse = analyse_archive(archive)
    if (all(is.na(analyse))) {
        add_journal_old(
            sprintf("Trouble with name format of archive %s. Nothing is done.", archive),
            verbose = TRUE
        )
        return(NULL)
    }    
    if (! analyse["extension"] %in% c("7z", "7z.001")) {
        add_journal_old(
            sprintf("Bad format of archive: %s. Should be in [%s]. Nothing is done.",
                paste(c("7z", "7z.001"), collapse = ", "),
                analyse["extension"]),
            verbose = TRUE
        )
        return(NULL)
    }
    if (! analyse["data"] == "RPG") {
        add_journal_old(
            sprintf("Bad type of data: %s. Should be RPG. Nothing is done.", analyse["data"]),
            verbose = TRUE
        )
        return(NULL)
    }
    if (! analyse["version"] %in% c("1-0", "2-0")) {
        add_journal_old(
            sprintf("Bad version of archive: %s. Should be 1-0 or 2-0. Nothing is done.", analyse["version"]),
            verbose = TRUE
        )
        return(NULL)
    }
    if (analyse["scope"] == "FR") {
        admissible = format_scope(floodam.data::department[floodam.data::department[["code"]] < 96, "code"])
    } else {
        admissible = format_scope(
            floodam.data::department[
                format_scope(floodam.data::department[["region"]], scope = "region") == 
                    format_scope(analyse["scope"], scope = "region"),
                "code"
            ]
        )
    }
    if (is.null(department)) {
        department = admissible
    } else {
        missing = format_scope(department)[! format_scope(department) %in% admissible]
        if (length(missing) > 0) {
            add_journal_old(
                sprintf(
                    "Some departments cannot be found in scope %s: %s", 
                    analyse["scope"],
                    paste(missing, collapse = ", ")
                ),
                verbose = TRUE
            )            
        }
        department = format_scope(department)[format_scope(department) %in% admissible]
    }
    if (length(department) == 0) {
        add_journal_old(sprintf("No department found in scope %s. Nothing is done", analyse["scope"]), verbose = TRUE)
        return(NULL)
    }

    # Start journal information
    dir.create(destination, showWarnings = FALSE, recursive = TRUE) # Check if error may be generated.
    if (journal == TRUE) {
        journal = file.path(
            destination,
            basename_core(archive)
        )
        init_log_info(
            journal,
            main = sprintf("%s in %s", basename(archive), origin),
            treatment = "adapt_rpg",
        )
    } else {
        journal = FALSE
    }
    
    # Decompression of archive
    if (analyse["version"] == "2-0") layer = "PARCELLES_GRAPHIQUES"
    if (analyse["version"] == "1-0") layer = "ILOTS_ANONYMES"
    path_temp = add_log_info(archive, shp_from_7z,
        info  = list(
            "short" = "decompressing",
            "long" = "Decompression of archive"
        ),
        journal = journal, verbose = verbose
    )

    on.exit(unlink(path_temp, recursive = TRUE, force = TRUE), add = TRUE)
    path_temp = file.path(tempdir(), analyse["year"], analyse["scope"])
    command_extract = sprintf(
        "7z e -aoa %s -o%s %s* -r",
        file.path(origin, archive), path_temp, layer)

    try_extract = try(
        system(command_extract, ignore.stdout = TRUE, ignore.stderr = TRUE),
        silent = TRUE)

    if (length(list.files(path_temp, ".shp")) == 0) {
        journal = add_journal_old(
            sprintf("Aborting, no %s shapefiles found in %s.", layer, archive), 
            journal = journal, verbose = verbose)
        unlink(path_temp, recursive = TRUE)
        return(NULL)
    }
    
    ### Read file
    # From here, it is supposed that file corresponds to RPG version 2.0
    input = suppressWarnings(
        sf::st_read(
            file.path(path_temp, list.files(path_temp, sprintf("%s.shp", layer))),
            stringsAsFactors = FALSE, quiet = TRUE)
    )

    journal = add_journal_old(
        sprintf("Layer %s successfully loaded.", layer),
        journal = journal, verbose = verbose)

    ### Manage data format
    if (is.null(scheme) && analyse["version"] == "2-0")  scheme = floodam.data::scheme_rpg_2
    if (is.null(scheme) && analyse["version"] == "2-0")  scheme = floodam.data::scheme_rpg_1
    geometry = sf::st_geometry(input)
    sf::st_geometry(input) = NULL
    sub_scheme = scheme[as.logical(scheme[["keep"]]) & scheme[["layer"]] == layer, ]
    # Keep only data in scheme, rename data as in scheme
    input = input[sub_scheme[["name_origin"]]]
    names(input) = sub_scheme[["name"]]
    # format factors
    selection = sub_scheme[["name"]][sub_scheme[["type"]] == "factor"]
    input[selection] = lapply(input[selection], as.factor)
    # format numeric
    selection = sub_scheme[["name"]][sub_scheme[["type"]] == "numeric"]
    input[selection] = lapply(input[selection], as.numeric)
    # format integer
    selection = sub_scheme[["name"]][sub_scheme[["type"]] == "integer"]
    input[selection] = lapply(input[selection], as.integer)
    # Add information to journal
    test = sapply(input, class) == sub_scheme[["type"]]
    if (all(test)) {
        journal = add_journal_old("All variables have the good type.", journal = journal, verbose = verbose)
    } else {
        journal = add_journal_old(
            sprintf("Some variables do not have the good type. Check %s.",
                paste("'", names(test)[!test], "'", sep = "", collapse = ", ")),
            journal = journal, verbose = verbose)
    }
    sf::st_geometry(input) = geometry
    rm(geometry)

    ### Manage projection
    # Check if any projection, if not put default one
    if (is.na(sf::st_crs(input))) {
        sf::st_crs(input) = analyse_projection(analyse["projection"])
        journal = add_journal_old(
            sprintf("No projection. Set to EPSG:%s.", analyse_projection(analyse["projection"])),
            journal = journal, verbose = verbose)            
    }
    # Transform projection if asked
    if (!is.null(projection)) {
        temp = try(sf::st_crs(projection), silent = TRUE)
        projection = if ("crs" %in% class(temp)) temp else projection
        if ("crs" %in% class(projection)) {
            input = sf::st_transform(input, crs = projection)
            journal = add_journal_old(
                sprintf("Projection %s applied.", projection[["input"]]),
                journal = journal, verbose = verbose)
        } else {
            journal = add_journal_old(
                sprintf(" Projection %s could not be applied.", projection),
                journal = journal, verbose = verbose)
        }
    }
    projection = sf::st_crs(input)

    ### Make geometry valid
    area_initial = sum(sf::st_area(input))
    input = sf::st_make_valid(input)
    geometry_type = sf::st_geometry_type(input)
    selection = geometry_type %in% c("MULTIPOLYGON", "POLYGON")
    input = suppressWarnings(sf::st_cast(input[selection, ], "MULTIPOLYGON"))
    area_final = sum(sf::st_area(input))
    diff.area = round((area_final - area_initial) / area_initial, 10)
    unselected = length(geometry_type) - sum(selection)
    journal = add_journal_old(
        sprintf("Geometry corrected (diff in area %s, unselected %s).", diff.area, unselected),
        journal = journal, verbose = verbose)

    ### Intersect with department
    scope = sf::st_transform(readRDS(dep_path), projection)
    scope = scope[scope[["department"]] %in% format_scope(department, type = "insee"), ]
    intersect = suppressMessages(sf::st_intersects(scope, input))
    names(intersect) = format_scope(scope[["department"]])
    journal = add_journal_old(
        sprintf("Intersected by %s departments", length(department)),
        journal = journal, verbose = verbose
    )

    result = list()
    for (dep in department) {
        result[[dep]] = input[intersect[[dep]], ]
    ### Add information from eaip (by departments)
        if (! is.null(eaip_path) && length(list.files(eaip_path, sprintf("%s.rds", dep))) == 1) {
            eaip = sf::st_transform(
                readRDS(file.path(eaip_path, list.files(eaip_path, sprintf("%s.rds", dep)))),
                projection
            )
            result[[dep]][["eaip"]] = in_layer(result[[dep]], eaip)
            journal = add_journal_old(
                sprintf("\tdepartment %s: eaip analysis successfully included for.", dep),
                journal = journal, verbose = verbose)
        } else {
            result[[dep]][["eaip"]] = NA
            journal = add_journal_old(
                sprintf("department %s: eaip analysis not included for department. NA added to variable eaip", dep),
                journal = journal, verbose = verbose)
        }
    ### Write data (by departments)
        saveRDS(result[[dep]], 
            file.path(destination, sprintf("%s_%s_%s.rds", tolower(analyse["data"]), analyse["year"], dep)))
        journal = add_journal_old(
            sprintf("department %s: successfully saved.", dep),
            journal = journal, verbose = verbose)
    }
        
    ### Retrieve result
    if (retrieve) {
        return(invisible(result))
    }
}