#' @title Manage a journal for follow-up of tasks
#'
#' @description
#' The function manages the log information for tasks that need to be linked to
#' each other. While the information is stored as a data.frame, it is presented
#' or saved as formatted character when asked to be presented (for instance when
#' verbose is set to TRUE).
#'
#' @details
#' If the parameter 'journal' is missing, `add_journal()` creates an
#' empty journal (a data.frame with the expected format). The data.frame have
#' 5 columns: 'start', 'end', 'task', 'status', 'degree', and 'elapsed'.
#'
#' If the parameter 'task' is not missing, and if no 'status' is given,
#' `add_journal()` adds a new line to journal, and sets 'start' to
#' current time, 'task' to given 'task'.
#'
#' If the parameter 'status' is not missing, `add_journal()` try to find
#' to which 'task' this is relevant. This can be done by giving explictly the
#' 'id', or by ginving the 'task'. If nothing is given, it is supposed that it
#' corresponds with the the last task with missing 'status'. For found 'task', 
#' it sets 'status' to given 'status', 'end' to current time. 'elapsed' to the
#' formatted difftime between 'end' and 'start' with \code{floodam.data::fmt}.
#'
#' If verbose == TRUE, a message is sent. If 'task' is given, a new line is
#' sent, but not ended by 'EOL'. When 'status' is given, message depend on what
#' was sent just before. If the last message was announcing the start of the
#' current task, it is completed with status and elapsed time. If not, a new
#' line is sent with the whole information (start, task, status, elapsed).
#' 'degree' is used to have some tabular formatting. The message sent is
#' formatted by \code{floodam.date:::format_journal}.
#'
#' @param journal data.frame, journal to be amended. May be missing (see
#'  details).
#' @param task character, description of the task. May be missing (see
#'  details).
#' @param status character, description of how the task has been performed. May
#'  be missing (see details).
#' @param id integer, row of the task to be amended. May be missing (see
#'  details).
#' @param degree integer, indicates how deep is the task
#' @param verbose logical, should a message be sent. Default to TRUE.
#'
#' @return a data frame of journal information.
#'
#' @export
#'
#' @encoding UTF-8
#' @author Frédéric Grelot
#'
#' @examples
#'
#' add_journal()
#' add_journal(status = "Nothing to be done...", verbose = FALSE)
#' {
#' journal = add_journal(
#'   task = sprintf(
#'     "maintain gaspar with floodam.data (%s)",
#'     utils::packageVersion("floodam.data")
#'   )
#' )
#' Sys.sleep(.1)
#' journal = add_journal(journal, task = "download_gaspar", degree = 1)
#' journal = add_journal(journal, task = "adapt_gaspar", degree = 1)
#' Sys.sleep(.1)
#' journal = add_journal(journal, task = "alert_gaspar", degree = 1)
#' journal = add_journal(journal, status = "Done", id = 2)
#' Sys.sleep(.1)
#' journal = add_journal(journal, status = "Done", task = "alert_gaspar")
#' journal = add_journal(journal, status = "Done")
#' journal = add_journal(journal, status = "Done")
#' journal
#' }

add_journal = function(
    journal,
    task,
    status,
    id,
    degree = 0,
    verbose = TRUE
) {
    if (missing(journal)) {
        journal = data.frame(
            task = character(),
            status = character(),
            start = as.POSIXct(NULL),
            end = as.POSIXct(NULL),
            elapsed = character()
        )
    }
    if (!missing(status)) {
        if (nrow(journal) == 0) {
            warning("Trying to add status information to an empty journal.")
            return(journal)
        }
        if (missing(id)) {
            if (!missing(task)) {
                id = suppressWarnings(max(which(journal[["task"]] == task)))
                if (id < 0) {
                    id = suppressWarnings(
                        max(which(is.na(journal[["status"]])))
                    )
                }
            } else {
                id = suppressWarnings(
                    max(which(is.na(journal[["status"]])))
                )
            }
            if (id < 0) {
                id = nrow(journal)
                warning("Trying to add status information with bad id.")               
            }
        }
        journal[id, "status"] = as.character(status)
        journal[id, "end"] = Sys.time()
        journal[id, "elapsed"] =
            floodam.data::fmt(journal[id, "end"] - journal[id, "start"])
        if (verbose == TRUE) {
            last = max(journal[["start"]], journal[-id, "end"], na.rm = TRUE)
            if (journal[id, "start"] == last) {
                message(
                    sprintf(
                        " %s [%s]",
                        journal[id, "status"],
                        journal[id, "elapsed"]
                    ),
                    appendLF = all(!is.na(journal[["status"]]))
                )
            } else {
                message(
                    sprintf(
                        "\n%s",
                        format_journal(journal[id, ])
                    ),
                    appendLF =  all(!is.na(journal[["status"]]))
                )
            }
        }
    } else if (!missing(task)) {
        information = data.frame(
            start = Sys.time(),
            end = as.POSIXct(NA),
            task = as.character(task),
            status = as.character(NA),
            degree = as.integer(degree),
            elapsed = as.character(NA)
        )
        journal = rbind(journal, information)
        if (verbose == TRUE) {
            id = nrow(journal)
            message(
                sprintf("\n%s", format_journal(journal[id, ])),
                appendLF = FALSE
            )
        }
    }
    return(journal)
}

write_journal = function(journal, path = "") {
    type = if (path == "") "log" else tools::file_ext(path)
    msg  = format_journal(journal)
    switch(
        type,
        "csv" = utils::write.table(
            journal,
            file = path,
            row.names = FALSE,
            sep = ",", dec = ".", qmethod = "double",
            col.names = !file.exists(path),
            append = file.exists(path)
        ),
        "log" = write(msg, file = path, append = file.exists(path)),
        warning(
            sprintf("write_journal: bad type (%s). Nothing is done.", type)
        )
    )
    return(invisible(msg))
}

format_journal = function(journal) {
    result = character()
    for (i in seq(nrow(journal))) {
        id = tools::file_ext(rownames(journal[i, ]))
        if (id == "") id = rownames(journal[i, ])
        tab = if (journal[i, "degree"] == 0) {
            ""
        } else  {
            paste(rep("\t", journal[i, "degree"]))
        }
        if (is.na(journal[i, "status"])) {
            msg = sprintf(
                "%s: [%s] %s%s...",
                journal[i, "start"],
                id,
                tab,
                journal[i, "task"]
            )
        } else {
            msg = sprintf(
                "%s: [%s] %s%s... %s [%s]",
                journal[i, "start"],
                id,
                tab,
                journal[i, "task"],
                journal[i, "status"],
                journal[i, "elapsed"]
            )
        }
        result[i] = msg
    }
    return(result)
}