#' @title Apply a step to data analysis
#' 
#' @description
#' This function allows to embed a analysis made by a function, so that log
#' information can be saved in a consistent way, messages can be sent depending
#' on the parameter verbose.
#'
#' @param x data to be adapted.
#' @param fun function, the analysis that may performed on x.
#' @param info list, gives the name of the step both in 'short' and 'long'
#' terms.
#' @param journal either FALSE, if not must be a character of length one, and
#' will be interpreted at the log file where to save any log information. Its
#' format should be in '*.log' in order to create also a '*-log.csv' log file
#' that will store information on time and any given verification in csv format.
#' @param verbose logical, should the some messages be sent in real time.
#' @param verification expression, a simple treatment that will be added to
#' *log.csv, may be missing.
#' @param level integer, the number of tabulation that will be added to the
#' message.
#' @param ... anything that may be needed by fun.
#' 
#' @return x transformed by fun
#'
#' @export
#'
#' @encoding UTF-8
#' @author Frédéric Grelot

add_log_info = function(
    x,
    fun,
    info,
    journal,
    verbose,
    verification,
    level = 1,
    ...
) {
    level = as.integer(level)
    if (is.na(level)) level = 1
    level = paste(rep("\t", level), collapse = "")

    if (verbose == TRUE) cat(sprintf("%s- %s... ", level, info[["long"]]))

    start = Sys.time()

    result = fun(x, ...)

    if (
        journal != FALSE &&
        ! missing(verification) &&
        methods::is(verification, "expression")
    ) {
        value = eval(verification)
        if (class(value) %in% "object_size") value = as.numeric(value)
        verification = as.character(verification)
    } else {
        value = NA
        verification = "none"
    }
    
    elapsed = Sys.time() - start
    duration = sprintf("(%s %s)", round(elapsed, 2), attr(elapsed, "units"))
    time = round(as.numeric(elapsed, units = "secs"), 3)

    if (journal != FALSE) {
        msg = sprintf("%s- %s: done %s.", level, info[["short"]], duration)
        write(msg, journal, append = TRUE)

        utils::write.table(
            data.frame(
                step = info[["short"]],
                time = time,
                verification = verification,
                value = value
            ),
            gsub(".log", "-log.csv", journal),
            sep = ",",
            dec = ".",
            row.names = FALSE,
            col.names = FALSE,
            append = TRUE
        )
    }

    if (verbose == TRUE) cat(sprintf("done %s\n", duration))

    return(result)
}

init_log_info = function(
    journal,
    main,
    treatment,
    add_sf = TRUE,
    add_lwgeom = TRUE
) {
    log_csv = gsub(".log", "-log.csv", journal)
    write(main, journal)
    write(
        sprintf(
            "\t- %s with floodam.data (%s)",
            treatment,
            utils::packageVersion("floodam.data")
        ),
        journal,
        append = TRUE
    )
    if (add_sf == TRUE) {
        write(sprintf("\t- %s", info_sf()), journal, append = TRUE)
    }
    if (add_lwgeom == TRUE) {
        write(sprintf("\t- %s", info_lwgeom()), journal, append = TRUE)
    }

    if (log_csv != journal) {
        write("step,duration,verification,value", log_csv)
    } else {
        msg = sprintf(
            "%s cannot generate '*-log.csv', no such file created.",
            journal
        )
        warning(msg)
    }
}

info_sf = function() {
    info = sf::sf_extSoftVersion()
    info = paste(
        c(names(info), "s2"),
        c(info, sf::sf_use_s2()),
        sep = ":", collapse = ", "
    )
    sprintf("sf (%s): %s", utils::packageVersion("sf"), info)
}

info_lwgeom = function() {
    info = lwgeom::lwgeom_extSoftVersion()
    info = paste(names(info), info, sep = ":", collapse = ", ")
    sprintf("lwgeom (%s): %s", utils::packageVersion("lwgeom"), info)
}

info_dim = function(x) {
    paste(dim(x), collapse = "\u00D7")
}