#' @title Create an interval of date
#'
#' @description
#' Create an interval of date from a vector
#' 
#' @param ... Anything that will be coerced to a character. See details for
#' format.
#'
#' @return  A vector of two dates of format "\%Y-\%m-\%d"
#' 
#' @details
#' Inputs are transformed in character vector, that we call x. When cretaed, x
#' shall be vector of characters with date in format "\%Y", "\%Y-\%m", or
#' "\%Y-\%m-%\d". It takes the minimum and maximum. min(x) is
#' completed, if necessary, as to be the beginning of period "\%Y" becomes
#' "\%Y-01-01", "\%Y-\%m" becomes "\%Y-\%m-01", while max(x) is completed, if
#' necessary, as to be the maximum of period "\%Y" becomes "\%Y-12-31",
#' "\%Y-\%m" becomes "\%Y-\%m-31" (or "..-28", "..-29", "..-30" depending on the
#' month).
#'
#' @export
#'
#' @examples
#' create_date(2020)
#' create_date(1995:2000)
#' create_date(1995, 2000)
#' create_date("1995", "2000-02")
#' create_date("2000-02")
#' create_date("2000-02", "2000-02-15")
#' create_date("2000-02-15")
#' \dontrun{create_date("toto")}

create_date = function(...) {
    x = as.character(unlist(list(...)))
    if (length(x) == 0) return(NULL)
    x = c(min(x), max(x))
    x[1] = gsub("^(\\d{4})$", "\\1-01-01", x[1])
    x[2] = gsub("^(\\d{4})$", "\\1-12-31", x[2])
    x[1] = gsub("^(\\d{4}-\\d{2})$", "\\1-01", x[1])
    if (length(grep("^\\d{4}-\\d{2}$", x[2]))) {
        x[2] = as.character(
            utils::tail(
                seq(
                    as.Date(
                        gsub(
                            "^(\\d{4}-\\d{2})$",
                            "\\1-01",
                            x[2]
                        )
                    ),
                    length = 2,
                    by = "months"
                ) - 1,
                1
            )
        )
    }
    if ("try-error" %in% class(try(as.Date(x), silent = TRUE)))
        stop(
            "Bad format for input, ",
            "should be a character of format %Y, %Y-%m or %Y-%m-%d."
        )
    return(x)
}