#' Download distant archives
#'
#' Core function used to downlaod archives
#' 
#' This function is used by other more specific functions. It takes as input a
#' to_do data.frame giving correspondance between origin urls to local
#' destinations. It also to select some types of archives to be downloaded. On
#' the good construction of the to_do data.frame depends the success of the
#' downloads!
#'
#' @param to_do data.frame. Two columns of character are expected.
#' @param extension character, vector of acceptable types of archive to be
#' downloaded.
#' @param name character, vector of acceptable names fo archive to be
#' downloaded
#' @param date character, something like a date that should be used as a
#' filter.
#' @param version character, something like a version that should be used as a
#' filter.
#' @param html character, indicates if the page is "simple" or is coming from
#' "data.gouv". This has implications on how to look for links.
#' @param verbose logical, should the function send somme messages while
#' running.
#'
#' @return nothing
#'
#' @export
#'
#' @examples
#' \dontrun{
#' destination = tempdir()
#' to_do_cquest = data.frame(
#'  origin = "https://data.cquest.org/registre_parcellaire_graphique/2018",
#'  destination = destination,
#'  short = "rpg_cquest",
#'  stringsAsFactors = FALSE
#' )
#' download_archive(to_do_cquest, version = "D976", verbose = TRUE)
#' 
#' to_do_ign = data.frame(
#'  origin = "https://geoservices.ign.fr/rpg",
#'  destination = destination,
#'  short = "rpg_ign",
#'  stringsAsFactors = FALSE
#' )
#' download_archive(to_do_ign, version = "D976", date = "2020", verbose = TRUE)
#' download_archive(to_do_ign, version = "D976", verbose = TRUE)
#' 
#' to_do = rbind(to_do_cquest, to_do_ign)
#' download_archive(to_do, version = "D976", verbose = TRUE)
#' }

download_archive = function(
    to_do,
    extension = c("gz", "zip", "7z"),
    name = NULL,
    date = NULL,
    version = NULL,
    html = "simple",
    verbose = FALSE
) {
    result = list()
    for (i in seq(nrow(to_do))) {
        origin = to_do[i, "origin"]
        journal = add_journal(
            task = sprintf(
                "download_archive from %s with floodam.data (%s)",
                origin,
                utils::packageVersion("floodam.data")
            ),
            verbose = verbose
        )

        archive_list = try(
            suppressWarnings(
                get_archive(
                    origin,
                    extension = extension, 
                    name = name, 
                    date = date, 
                    version = version,
                    html = html
                )
            ), 
            silent = TRUE
        )
        if (methods::is(archive_list, "try-error")) {
            journal = add_journal(
                journal,
                status = "Aborted (no such url)",
                verbose = verbose
            )
        } else if (length(archive_list) == 0) {
            msg = sprintf(
                "(no archive with name=%s, version=%s, date=%s, extension=%s)",
                if (is.null(name)) "NULL" else name,
                if (is.null(version)) "NULL" else version,
                if (is.null(date)) "NULL" else date,
                if (is.null(extension)) "NULL" else extension
            )
            journal = add_journal(
                journal,
                status = sprintf("Aborted %s", msg),
                verbose = verbose
            )
        } else {
            message(
                sprintf(" %s archives in progress...", length(archive_list)),
                appendLF = FALSE
            )
            dir.create(
                to_do[i, "destination"],
                showWarnings = FALSE,
                recursive = TRUE
            )
            success = structure(integer(2), names = c("OK", "KO"))
            for (file_origin in archive_list) {
                archive = basename(file_origin)
                file_destination = file.path(to_do[i, "destination"], archive)
                journal = add_journal(
                    journal,
                    task = sprintf("downloading %s", archive),
                    degree = journal[1, "degree"] + 1,
                    verbose = verbose
                )
                
                try_download = try(utils::download.file(
                    url =  file_origin, 
                    destfile = file_destination,
                    method = "wget", quiet = TRUE),
                    silent = TRUE)
                if (methods::is(try_download, "try-error")) {
                    unlink(file_destination)
                    success["KO"] = success["KO"] + 1
                    journal = add_journal(
                        journal,
                        status = "Unsuccessful",
                        verbose = verbose
                    )
                } else {
                    success["OK"] = success["OK"] + 1
                    journal = add_journal(
                        journal,
                        status = "Successful",
                        verbose = verbose
                    )
                }
            }
            journal = add_journal(
                journal,
                status = sprintf(
                    "Done (%s archives treated, %s OK, %s KO)",
                    sum(success),
                    success["OK"],
                    success["KO"]
                ),
                verbose = verbose
            )
            result[[origin]] = journal
            write_journal(
                journal,
                file.path(
                    to_do[i, "destination"],
                    sprintf("%s.%s", gsub("/", "_", to_do[i, "short"]), "log")
                )
            )
        }
    }
    return(invisible(do.call(rbind, result)))
}