#' @title Download gaspar database
#' 
#' @description
#' 
#' `download_gaspar()` downloads gaspar database from origin to destination.
#' 
#' @details
#' 
#' The origin of the data is set by default to
#'      \url{https://files.georisques.fr}. It can also be downloaded from
#'      "cquest" and "opendatarchives" mirrors.
#'
#' The georisques provider gives no indication of the time of the update of
#'      gaspar db. So, if origin does not contain an indication of current day
#'      in the format "\%Y-\%m-\%d", this information will be appended to
#'      destination as a subirectory of destination.
#' 
#' The destination directory will be created by the function, if missing.
#'
#' @param destination character, the address where dara are stocked
#' @param origin character, either a keyword or the address from where data are
#'  downloaded. Best to keep it at default value.
#' @param name character, vector of acceptable names fo archive to be
#'  downloaded
#' @param extension character, vector of acceptable types of archive to be
#'  downloaded
#' @param verbose logical, should the function send somme messages while
#'  running
#'
#' @return If everything got fine the `download_gaspar()` returns the
#'  integer 0, if not it returns an object of class "try-error".
#'
#' @export
#'
#' @examples
#' 
#' destination = tempfile()
#' \dontrun{download_gaspar(destination)}
#' \dontrun{download_gaspar(destination, origin = "cquest")}
#' \dontrun{download_gaspar(tempfile, origin = "opendatarchives")}

download_gaspar = function(
    destination,
    origin = "georisques",
    name = "gaspar",
    extension = "zip",
    verbose = TRUE) {

    send_message("download_gaspar...", appendLF = FALSE, verbose = verbose)
    on.exit(write(journal, file.path(destination, "gaspar.log"), append = TRUE))
    on.exit(
        send_message(
            gsub(sprintf("%s: download_gaspar", today), "", journal),
            verbose = verbose
        ),
        add = TRUE
    )

    today =  as.character(Sys.Date())
    archive = sprintf("%s.%s", name, extension)
    origin = switch(
        origin,
        "georisques" = file.path("https://files.georisques.fr/GASPAR", archive),
        "cquest" = file.path("https://data.cquest.org/georisques", archive),
        "opendatarchives" = file.path("http://files.opendatarchives.fr/georisques.gouv.fr", archive),
        file.path("https://files.georisques.fr/GASPAR", archive)
    )
    journal = sprintf("%s: download_gaspar failed '%s' with 'floodam.data' (%s)",
        today,
        origin,
        utils::packageVersion("floodam.data")
    )

    if (httr::HEAD(origin)[["all_headers"]][[1]][["status"]] != 200) {
        return(invisible(NULL))
    }

    dir.create(file.path(destination, today), showWarnings = FALSE, recursive = TRUE)
    result = utils::download.file(origin, destfile = file.path(destination, today, archive), method = "wget", quiet = TRUE)
    journal = sprintf("%s: download_gaspar successful '%s' to '%s' with 'floodam.data' (%s) ",
        today,
        origin,
        file.path(destination, today, archive),
        utils::packageVersion("floodam.data")
    )

    return(invisible(result))
}