% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/in_layer.R
\encoding{UTF-8}
\name{in_layer}
\alias{in_layer}
\title{Control if an object x is inside an object y}
\usage{
in_layer(
  x,
  y,
  treatment = "intersect",
  name = "in_layer",
  divide = TRUE,
  s2 = FALSE
)
}
\arguments{
\item{x}{object that is admissible for st_intersects.}

\item{y}{object that is admissible for st_intersects.}

\item{treatment}{character indicating which treatment should be done. See
details.}

\item{name}{character, name that will be used for results. Default to
"in_layer".}

\item{divide}{logical, should lwgeom::st_subdivide be used on y to have fast
results.}

\item{s2}{logical, should sf_use_s2 be used or not.}
}
\value{
If treatment = "intersect", logical with same length as nrow(x)
indicating whether geometry in x are (event partially) in y.
In any other case, data.frame whose column corresponds to asked treaments and
rows to geometry in x.
}
\description{
'in_layer' checks if geometries in 'x' are, even partially, inside any
geometry in 'y'.
}
\details{
When \code{treatment} includes "interesect", a logical is produced that
indicates if geometries in x intersects any geometries in y.
When \code{treatment} includes "intersection", a numeric is produced that
indicates for each geometry in x the area that is intersected by all
geometries in y.
When \code{treatment} includes "proportion", a numeric is produced that
indicates for each geometry in x the proportion of area that is intersected
by all geometries in y.

The use of \code{divide} should give fastet results in real case where y may
have a lot of details in its polygon defintion. As lwgeom::subdivide is
performed within the function and takes some time, it is a better strategy to
use \code{treatment} equals to "all" when both intersect and intersection are
asked, because it is done only one time for both treatments.

To have more consistent results st_union is performed on y each time
\code{treatment} includes either "intersection" or "proportion".
}
\examples{
fun_p1 = function(p, n, x, y, s = 0){
  fun = function(p, x, y, s) {p * s + c(x, y)}
  if (s == 0) s = 1 else s = runif(n) * s
  result = mapply(fun, runif(n) * x, runif(n) * y, s, MoreArgs = list(p = p), SIMPLIFY = FALSE)
  sf::st_sfc(result)
}
fun_p2 = function(n) {
  theta = seq(0, 1, length.out = n) * 2 * pi
  theta[n] =  theta[1]
  rho = .8 + runif(n) * .2
  rho[n] = rho[1]
  result = mapply(
    function(theta, rho) {c(sin(theta), cos(theta)) * rho},
    theta,
    rho,
    SIMPLIFY = FALSE
  )
  sf::st_polygon(list(do.call(rbind, result)))
}

set.seed(0)
p1 = fun_p2(100)
p2 = fun_p2(100)
x = fun_p1(p1 * .2, 100, 10, 10)
y = fun_p1(p1, 25, 10, 10, .5)

in_layer(x, y)
in_layer(y, x)
in_layer(x, y, "intersect")
in_layer(x, y, "intersection")
in_layer(x, y, "proportion")
in_layer(x, y, "all", "y")
in_layer(x, y, "all", "y", divide = FALSE)

\dontrun{
n = 5
z = sf::st_sfc(replicate(n, sf::st_union(fun_p1(p2 * .15, 1000, 10, 10, 0))))
z = sf::st_make_valid(sf::st_difference(z))

plot(z, col = scales::alpha(rainbow(n), .5), axes = TRUE)
plot(x, col = scales::alpha("blue", .5), add = TRUE)
plot(y, col = scales::alpha("red", .5), add = TRUE)

microbenchmark::microbenchmark(
  slow = in_layer(x, z, divide = FALSE),
  fast = in_layer(x, z, divide = TRUE),
  times = 10
)
microbenchmark::microbenchmark(
  slow = in_layer(x, z, "intersection", divide = FALSE),
  fast = in_layer(x, z, "intersection", divide = TRUE),
  times = 10
)
microbenchmark::microbenchmark(
  slow = in_layer(x, z, "all", divide = FALSE),
  fast = in_layer(x, z, "all", divide = TRUE),
  times = 10
)
}
}
\author{
Frédéric Grelot
}
